#define F_CPU 16000000  // 16 MHz

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <stdio.h>
#include <stdlib.h>

#include "board.h"


#define C_MOTOR 1

volatile char cmd = 0;
volatile char command = 0;
volatile unsigned char command1 = ' ';
volatile unsigned char command2 = ' ';

/**
 * UART interrupt - data received
 * format: 'A', 'B', '\n' (3 bytes)
 */
ISR(USART_RXC_vect) {
  cmd++;
  unsigned char rec = uartReceive();
  
  if (cmd == 1) {
    command1 = rec;
  } else if (cmd == 2) {
    command2 = rec;
  } else if (rec == '\n') {    
    command = 0;
    
    // LEFT MOTOR or other command
    switch (command1) {
      case 'F': leftSpeed(255); command = C_MOTOR; break;
      case 'S': leftSpeed(0); command = C_MOTOR; break;
      case 'B': leftSpeed(-255); command = C_MOTOR; break;
      case 'T': turnServo(command2); break;
      case '@': 
        // Bye message
        uartTransmit('b');
        uartTransmit('y');
        uartTransmit('e');
        uartTransmit('\n');
        
        // Save flag to eeprom
        eeprom_write_byte((unsigned char*)0x00, 0x01);
        
        wdt_enable(WDTO_30MS); // Watchdog reset
        
        while (1); // Never-ending loop
      break;
    }
    
    // RIGHT MOTOR
    if (command == C_MOTOR) {
      switch (command2) {
        case 'F': rightSpeed(255); break;
        case 'S': rightSpeed(0); break;
        case 'B': rightSpeed(-255); break;
      }
    }
    
    // RETURN "OK" ACK
    
    if (command <= C_MOTOR)  {    
     uartTransmit('O');
     uartTransmit('K');
	   uartTransmit('\n');
	  }
	  
	  cmd = 0;
	}
}


int main (void) {	
	initBoard(); // init hardware (GPIO, Timers, Interrupts, ADC, UART, ...)
	sei(); // enable interrupts
	_delay_ms(50); // wait some time before UART inits
	
	greenOn(); // green LED on
	
	uartPuts("\nRoboBoard\n\n"); // welcome message
  
  // SPEED - very small for 12V adapter
  setLowSpeed();
  
  // DIPSW3 == 1 => high speed (for batteries)
  if (DIPSW3) {
    setHighSpeed();
  }
  
  // LED blinking
  int i = 1;
  while (1) {
    if (i) {
      greenOn();
      i = 0;
    } else {
      greenOff();
      i = 1;
    }
    
    _delay_ms(200);
  }

	return 0;
}

