/**
 * Board definitions
 */

/* UART */
  #define BAUDRATE 19200
  #define MYUBRR F_CPU/16/BAUDRATE-1
  
  #include "lib/uart.c"
  
/* A/D Converter */
  #define ADC_VREF_TYPE 0x40
  #define adcInit() { ADMUX = ADC_VREF_TYPE; ADCSRA = 0x87; }
  #include "lib/adc.c"

/* GP2D120 (IR RANGER) */
  #define range() gp2d120ToCm(readAdc(6))
  #include "lib/gp2d120.c"

/* SERVO */
  #define SERVO_TIMER TCNT2
  #define SERVO_TIMER_OVF TIMER2_OVF_vect
  
  #define S1_LEFT 125
  #define S1_CENTER 80
  #define S1_RIGHT 35
  
  #define servoInit() { DDRA |= (1<<PA7);  TCCR2 = 0b00000001; OCR2 = 0xff; TCNT2 = 0; TIMSK = 0b01000000; }
  #define servoOn() { PORTA |= (1<<PA7); }
  #define servoOff() { PORTA &= ~(1<<PA7); }
  
  #include "lib/servo.c"

/* MOTORS */
  #define LOW_SPEED 405
  #define HIGH_SPEED 255

  // AC adapter settings (405 is the slowest, 250 is the fastest, the speed shouldn't be higher (OCR1A must be over 250))
  #define setLowSpeed() { mSpeedMax = LOW_SPEED; }
  
  // 7.2 V Battery settings (255 is the slowest)
  #define setHighSpeed() { mSpeedMax = HIGH_SPEED; }
  
  #include "lib/motors.c"

/* BUTTON */
  #define readButton() !(PINB & (1 << PB4)) // active in 0
  
  #define buttonInit() { PORTB |= (1<<PB4); } 

/* DIP SWITCH */
  #define DIPSW1 !(PINB & (1 << PB0)) // active in 0
  #define DIPSW2 !(PINB & (1 << PB1))
  #define DIPSW3 !(PINB & (1 << PB2))
  #define DIPSW4 !(PINB & (1 << PB3))
  
  #define dipInit() { PORTB |= (1<<PB0) | (1<<PB1) | (1<<PB2) | (1<<PB3); } 
  
/* LINE SENSOR */
  // 1 == left, 5 == right
  #define LINE5 !(PINA & (1 << PA0)) // active in 0
  #define LINE4 !(PINA & (1 << PA1))
  #define LINE3 !(PINA & (1 << PA2))
  #define LINE2 !(PINA & (1 << PA3))
  #define LINE1 !(PINA & (1 << PA4))
  
  #define BLACK(line) line == 0
  
  #define lineInit() { PORTA |= (1<<PA0) | (1<<PA1) | (1<<PA2) | (1<<PA3) | (1<<PA4); } 
  
/* LEDS */
  #define redOff() PORTC &= ~(1<<PC3);
  #define redOn() PORTC |= (1<<PC3);
  #define greenOff() PORTC &= ~(1<<PC2);
  #define greenOn() PORTC |= (1<<PC2);
  
  #define ledsInit() { DDRC |= (1<<PC3) | (1<<PC2); }
  
/* hardware init */
#define initBoard() { motorsInit(); ledsInit(); dipInit(); buttonInit(); servoInit(); uartInit(MYUBRR); adcInit(); lineInit(); }